=== added file 'bzrlib/inter.py'
--- bzrlib/inter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/inter.py	2006-03-02 01:26:22 +0000
@@ -0,0 +1,74 @@
+# Copyright (C) 2006 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Inter-object utility class."""
+
+
+class InterObject(object):
+    """This class represents operations taking place between two objects.
+
+    Its instances have methods like join or copy_content or fetch, and contain
+    references to the source and target objects these operations can be 
+    carried out between.
+
+    Often we will provide convenience methods on the objects which carry out
+    operations with another of similar type - they will always forward to
+    a subclass of InterObject - i.e. 
+    InterVersionedFile.get(other).method_name(parameters).
+    """
+
+    # _optimisers = set()
+    # Each concrete InterObject type should have its own optimisers set.
+
+    def __init__(self, source, target):
+        """Construct a default InterObject instance. Please use 'get'.
+        
+        Only subclasses of InterObject should call 
+        InterObject.__init__ - clients should call InterFOO.get where FOO
+        is the base type of the objects they are interacting between. I.e.
+        InterVersionedFile or InterRepository.
+        get() is a convenience class method which will create an optimised
+        InterFOO if possible.
+        """
+        self.source = source
+        self.target = target
+
+    @classmethod
+    def get(klass, source, target):
+        """Retrieve a Inter worker object for these objects.
+
+        :param source: the object to be the 'source' member of
+                       the InterObject instance.
+        :param target: the object to be the 'target' member of
+                       the InterObject instance.
+        If an optimised worker exists it will be used otherwise
+        a default Inter worker instance will be created.
+        """
+        for provider in klass._optimisers:
+            if provider.is_compatible(source, target):
+                return provider(source, target)
+        return klass(source, target)
+
+    @classmethod
+    def register_optimiser(klass, optimiser):
+        """Register an InterObject optimiser."""
+        klass._optimisers.add(optimiser)
+
+    @classmethod
+    def unregister_optimiser(klass, optimiser):
+        """Unregister an InterObject optimiser."""
+        klass._optimisers.remove(optimiser)

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-03-01 12:09:59 +0000
+++ bzrlib/knit.py	2006-03-02 01:26:22 +0000
@@ -97,6 +97,16 @@
 INDEX_SUFFIX = '.kndx'
 
 
+# convenience factories for testing or use:
+def AnnotatedKnitFactory(name, transport, mode=None):
+    """Create a knit with path name in transport transport."""
+    return KnitVersionedFile(transport,
+                             name,
+                             'w',
+                             KnitAnnotateFactory(),
+                             delta=True)
+
+
 class KnitContent(object):
     """Content of a knit version to which deltas can be applied."""
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-03-01 03:26:23 +0000
+++ bzrlib/repository.py	2006-03-02 01:26:22 +0000
@@ -23,6 +23,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 import bzrlib.errors as errors
 from bzrlib.errors import InvalidRevisionId
+from bzrlib.inter import InterObject
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.osutils import safe_unicode
 from bzrlib.revision import NULL_REVISION
@@ -966,7 +967,7 @@
                    RepositoryFormat6()]
 
 
-class InterRepository(object):
+class InterRepository(InterObject):
     """This class represents operations taking place between two repositories.
 
     Its instances have methods like copy_content and fetch, and contain
@@ -977,24 +978,10 @@
     operations with another repository - they will always forward to
     InterRepository.get(other).method_name(parameters).
     """
-    # XXX: FIXME: FUTURE: robertc
-    # testing of these probably requires a factory in optimiser type, and 
-    # then a test adapter to test each type thoroughly.
-    #
 
     _optimisers = set()
     """The available optimised InterRepository types."""
 
-    def __init__(self, source, target):
-        """Construct a default InterRepository instance. Please use 'get'.
-        
-        Only subclasses of InterRepository should call 
-        InterRepository.__init__ - clients should call InterRepository.get
-        instead which will create an optimised InterRepository if possible.
-        """
-        self.source = source
-        self.target = target
-
     @needs_write_lock
     def copy_content(self, revision_id=None, basis=None):
         """Make a complete copy of the content in self into destination.
@@ -1053,22 +1040,6 @@
                         pb=pb)
         return f.count_copied, f.failed_revisions
 
-    @classmethod
-    def get(klass, repository_source, repository_target):
-        """Retrieve a InterRepository worker object for these repositories.
-
-        :param repository_source: the repository to be the 'source' member of
-                                  the InterRepository instance.
-        :param repository_target: the repository to be the 'target' member of
-                                the InterRepository instance.
-        If an optimised InterRepository worker exists it will be used otherwise
-        a default InterRepository instance will be created.
-        """
-        for provider in klass._optimisers:
-            if provider.is_compatible(repository_source, repository_target):
-                return provider(repository_source, repository_target)
-        return InterRepository(repository_source, repository_target)
-
     def lock_read(self):
         """Take out a logical read lock.
 
@@ -1107,11 +1078,6 @@
         # that we've decided we need.
         return [rev_id for rev_id in source_ids if rev_id in result_set]
 
-    @classmethod
-    def register_optimiser(klass, optimiser):
-        """Register an InterRepository optimiser."""
-        klass._optimisers.add(optimiser)
-
     def unlock(self):
         """Release the locks on source and target."""
         try:
@@ -1119,11 +1085,6 @@
         finally:
             self.source.unlock()
 
-    @classmethod
-    def unregister_optimiser(klass, optimiser):
-        """Unregister an InterRepository optimiser."""
-        klass._optimisers.remove(optimiser)
-
 
 class InterWeaveRepo(InterRepository):
     """Optimised code paths between Weave based repositories."""

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-03-01 12:09:59 +0000
+++ bzrlib/tests/__init__.py	2006-03-02 01:26:22 +0000
@@ -84,6 +84,7 @@
     import bzrlib.tests.branch_implementations
     import bzrlib.tests.bzrdir_implementations
     import bzrlib.tests.interrepository_implementations
+    import bzrlib.tests.interversionedfile_implementations
     import bzrlib.tests.repository_implementations
     import bzrlib.tests.workingtree_implementations
     return [
@@ -92,6 +93,7 @@
             bzrlib.tests.branch_implementations,
             bzrlib.tests.bzrdir_implementations,
             bzrlib.tests.interrepository_implementations,
+            bzrlib.tests.interversionedfile_implementations,
             bzrlib.tests.repository_implementations,
             bzrlib.tests.workingtree_implementations,
             ]

=== added directory 'bzrlib/tests/interversionedfile_implementations'
=== added file 'bzrlib/tests/interversionedfile_implementations/__init__.py'
--- bzrlib/tests/interversionedfile_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/interversionedfile_implementations/__init__.py	2006-03-02 01:26:22 +0000
@@ -0,0 +1,51 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""InterVersioned implementation tests for bzr.
+
+These test the conformance of all the interversionedfile variations to the
+expected API including generally applicable corner cases.
+Specific tests for individual cases are in the tests/test_versionedfile.py file 
+rather than in tests/interversionedfile_implementations/*.py.
+"""
+
+from bzrlib.versionedfile import InterVersionedFileTestProviderAdapter
+                            
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_interversionedfile_implementations = [
+        'bzrlib.tests.interversionedfile_implementations.test_join',
+        ]
+    adapter = InterVersionedFileTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        InterVersionedFileTestProviderAdapter.default_test_list()
+        )
+    loader = TestLoader()
+    adapt_modules(test_interversionedfile_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/interversionedfile_implementations/test_join.py'
--- bzrlib/tests/interversionedfile_implementations/test_join.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/interversionedfile_implementations/test_join.py	2006-03-02 01:26:22 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for join between versioned files."""
+
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.transport import get_transport
+
+
+class TestJoin(TestCaseWithTransport):
+    #Tests have self.versionedfile_factory and self.versionedfile_factory_to
+    #available to create source and target versioned files respectively.
+
+    def get_source(self, name='source'):
+        """Get a versioned file we will be joining from."""
+        return self.versionedfile_factory(name,
+                                          get_transport(self.get_url()))
+
+    def get_target(self, name='source'):
+        """"Get an empty versioned file to join into."""
+        return self.versionedfile_factory_to(name,
+                                             get_transport(self.get_url()))
+
+    def test_join(self):
+        f1 = self.get_source()
+        f1.add_lines('r0', [], ['a\n', 'b\n'])
+        f1.add_lines('r1', ['r0'], ['c\n', 'b\n'])
+        f2 = self.get_target()
+        f2.join(f1, None)
+        def verify_file(f):
+            self.assertTrue(f.has_version('r0'))
+            self.assertTrue(f.has_version('r1'))
+        verify_file(f2)
+        verify_file(self.get_target())
+
+        self.assertRaises(RevisionNotPresent,
+            f2.join, f1, version_ids=['r3'])
+
+        #f3 = self.get_file('1')
+        #f3.add_lines('r0', ['a\n', 'b\n'], [])
+        #f3.add_lines('r1', ['c\n', 'b\n'], ['r0'])
+        #f4 = self.get_file('2')
+        #f4.join(f3, ['r0'])
+        #self.assertTrue(f4.has_version('r0'))
+        #self.assertFalse(f4.has_version('r1'))
+

=== added file 'bzrlib/tests/test_reweave.py'
--- bzrlib/tests/test_reweave.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_reweave.py	2006-03-02 01:26:22 +0000
@@ -0,0 +1,36 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test reweave code.
+
+Reweave takes two weaves containing a partial view of history and combines
+them into a single weave containing all the information.  This can include 
+
+ - versions recorded in only one file
+
+ - versions with different (but not contradictory) lists of parent 
+   revisions
+
+It is an error if either of these conditions occur:
+
+ - contradictory ancestry graphs, e.g.
+   - v1 is an ancestor of v2 in one weave, and vice versa in the other
+   - different text for any version 
+"""
+
+import os
+import sys
+

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-02-20 07:01:10 +0000
+++ bzrlib/tests/test_selftest.py	2006-03-02 01:26:22 +0000
@@ -334,6 +334,34 @@
         self.assertEqual(tests[1].transport_readonly_server, server2)
 
 
+class TestInterVersionedFileProviderAdapter(TestCase):
+    """A group of tests that test the InterVersionedFile test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.versionedfile import InterVersionedFileTestProviderAdapter
+        input_test = TestInterRepositoryProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = [(str, "C1", "C2"), (str, "D1", "D2")]
+        adapter = InterVersionedFileTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].interversionedfile_class, formats[0][0])
+        self.assertEqual(tests[0].versionedfile_factory, formats[0][1])
+        self.assertEqual(tests[0].versionedfile_factory_to, formats[0][2])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[0].interversionedfile_class, formats[1][0])
+        self.assertEqual(tests[1].versionedfile_factory, formats[1][1])
+        self.assertEqual(tests[1].versionedfile_factory_to, formats[1][2])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
 class TestWorkingTreeProviderAdapter(TestCase):
     """A group of tests that test the workingtree implementation test adapter."""
 

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2006-03-01 12:09:59 +0000
+++ bzrlib/tests/test_versionedfile.py	2006-03-02 01:26:22 +0000
@@ -30,9 +30,10 @@
                            )
 from bzrlib.knit import KnitVersionedFile, \
      KnitAnnotateFactory
-from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.trace import mutter
 from bzrlib.transport.local import LocalTransport
+import bzrlib.versionedfile as versionedfile
 from bzrlib.weave import WeaveFile
 from bzrlib.weavefile import read_weave
 
@@ -126,29 +127,6 @@
         self.assertRaises(RevisionNotPresent,
             f.annotate, 'foo')
 
-    def test_join(self):
-        f1 = self.get_file('1')
-        f1.add_lines('r0', [], ['a\n', 'b\n'])
-        f1.add_lines('r1', ['r0'], ['c\n', 'b\n'])
-        f2 = self.get_file('2')
-        f2.join(f1, None)
-        def verify_file(f):
-            self.assertTrue(f.has_version('r0'))
-            self.assertTrue(f.has_version('r1'))
-        verify_file(f2)
-        verify_file(self.reopen_file('2'))
-
-        self.assertRaises(RevisionNotPresent,
-            f2.join, f1, version_ids=['r3'])
-
-        #f3 = self.get_file('1')
-        #f3.add_lines('r0', ['a\n', 'b\n'], [])
-        #f3.add_lines('r1', ['c\n', 'b\n'], ['r0'])
-        #f4 = self.get_file('2')
-        #f4.join(f3, ['r0'])
-        #self.assertTrue(f4.has_version('r0'))
-        #self.assertFalse(f4.has_version('r1'))
-
     def test_walk(self):
         f = self.get_file('1')
         f.add_lines('r0', [], ['a\n', 'b\n'])
@@ -312,7 +290,7 @@
         self.assertEqual(['a'], w1.get_parents('b'))
         self.assertEqual(['b'], w1.get_parents('c'))
 
-class TestWeave(TestCaseInTempDir, VersionedFileTestMixIn):
+class TestWeave(TestCaseWithTransport, VersionedFileTestMixIn):
 
     def get_file(self, name='foo'):
         return WeaveFile(name, LocalTransport('.'))
@@ -355,7 +333,7 @@
         return WeaveFile(name, LocalTransport('.'))
 
 
-class TestKnit(TestCaseInTempDir, VersionedFileTestMixIn):
+class TestKnit(TestCaseWithTransport, VersionedFileTestMixIn):
 
     def get_file(self, name='foo'):
         return KnitVersionedFile(LocalTransport('.'),
@@ -373,3 +351,70 @@
 
     def test_detection(self):
         print "TODO for merging: create a corrupted knit."
+
+
+class InterString(versionedfile.InterVersionedFile):
+    """An inter-versionedfile optimised code path for strings.
+
+    This is for use during testing where we use strings as versionedfiles
+    so that none of the default regsitered interversionedfile classes will
+    match - which lets us test the match logic.
+    """
+
+    @staticmethod
+    def is_compatible(source, target):
+        """InterString is compatible with strings-as-versionedfiles."""
+        return isinstance(source, str) and isinstance(target, str)
+
+
+# TODO this and the InterRepository core logic should be consolidatable
+# if we make the registry a separate class though we still need to 
+# test the behaviour in the active registry to catch failure-to-handle-
+# stange-objects
+class TestInterVersionedFile(TestCaseWithTransport):
+
+    def test_get_default_inter_versionedfile(self):
+        # test that the InterVersionedFile.get(a, b) probes
+        # for a class where is_compatible(a, b) returns
+        # true and returns a default interversionedfile otherwise.
+        # This also tests that the default registered optimised interversionedfile
+        # classes do not barf inappropriately when a surprising versionedfile type
+        # is handed to them.
+        dummy_a = "VersionedFile 1."
+        dummy_b = "VersionedFile 2."
+        self.assertGetsDefaultInterVersionedFile(dummy_a, dummy_b)
+
+    def assertGetsDefaultInterVersionedFile(self, a, b):
+        """Asserts that InterVersionedFile.get(a, b) -> the default."""
+        inter = versionedfile.InterVersionedFile.get(a, b)
+        self.assertEqual(versionedfile.InterVersionedFile,
+                         inter.__class__)
+        self.assertEqual(a, inter.source)
+        self.assertEqual(b, inter.target)
+
+    def test_register_inter_versionedfile_class(self):
+        # test that a optimised code path provider - a
+        # InterVersionedFile subclass can be registered and unregistered
+        # and that it is correctly selected when given a versionedfile
+        # pair that it returns true on for the is_compatible static method
+        # check
+        dummy_a = "VersionedFile 1."
+        dummy_b = "VersionedFile 2."
+        versionedfile.InterVersionedFile.register_optimiser(InterString)
+        try:
+            # we should get the default for something InterString returns False
+            # to
+            self.assertFalse(InterString.is_compatible(dummy_a, None))
+            self.assertGetsDefaultInterVersionedFile(dummy_a, None)
+            # and we should get an InterString for a pair it 'likes'
+            self.assertTrue(InterString.is_compatible(dummy_a, dummy_b))
+            inter = versionedfile.InterVersionedFile.get(dummy_a, dummy_b)
+            self.assertEqual(InterString, inter.__class__)
+            self.assertEqual(dummy_a, inter.source)
+            self.assertEqual(dummy_b, inter.target)
+        finally:
+            versionedfile.InterVersionedFile.unregister_optimiser(InterString)
+        # now we should get the default InterVersionedFile object again.
+        self.assertGetsDefaultInterVersionedFile(dummy_a, dummy_b)
+
+

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-03-01 12:09:59 +0000
+++ bzrlib/versionedfile.py	2006-03-02 01:26:22 +0000
@@ -24,6 +24,11 @@
 """Versioned text file storage api."""
 
 
+from copy import deepcopy
+from unittest import TestSuite
+
+
+from bzrlib.inter import InterObject
 from bzrlib.symbol_versioning import *
 
 
@@ -267,3 +272,71 @@
                 assert state in ('irrelevant', 'ghost-a', 'ghost-b', 'killed-base',
                                  'killed-both'), \
                        state
+
+
+class InterVersionedFile(InterObject):
+    """This class represents operations taking place between two versionedfiles..
+
+    Its instances have methods like join, and contain
+    references to the source and target versionedfiles these operations can be 
+    carried out on.
+
+    Often we will provide convenience methods on 'versionedfile' which carry out
+    operations with another versionedfile - they will always forward to
+    InterVersionedFile.get(other).method_name(parameters).
+    """
+
+    _optimisers = set()
+    """The available optimised InterVersionedFile types."""
+
+
+class InterVersionedFileTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple inter versioned-file classes.
+
+    This is done by copying the test once for each interversionedfile provider
+    and injecting the transport_server, transport_readonly_server,
+    versionedfile_factory and versionedfile_factory_to classes into each copy.
+    Each copy is also given a new id() to make it easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
+    
+    def adapt(self, test):
+        result = TestSuite()
+        for (interversionedfile_class,
+             versionedfile_factory,
+             versionedfile_factory_to) in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.interversionedfile_class = interversionedfile_class
+            new_test.versionedfile_factory = versionedfile_factory
+            new_test.versionedfile_factory_to = versionedfile_factory_to
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), interversionedfile_class.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
+    @staticmethod
+    def default_test_list():
+        """Generate the default list of interversionedfile permutations to test."""
+        from bzrlib.weave import WeaveFile
+        from bzrlib.knit import AnnotatedKnitFactory
+        result = []
+        # test the fallback InterVersionedFile from weave to annotated knits
+        result.append((InterVersionedFile, 
+                       WeaveFile,
+                       AnnotatedKnitFactory))
+        for optimiser in InterVersionedFile._optimisers:
+            result.append((optimiser,
+                           optimiser._matching_file_factory,
+                           optimiser._matching_file_factory
+                           ))
+        # if there are specific combinations we want to use, we can add them 
+        # here.
+        return result

